@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <errno.h>
 #include <sys/types.h>
+#include <fcntl.h>
 
 #include "log.h"
 #include "lxc.h"
@@ -41,28 +42,43 @@ static struct lxc_list defines;
 
 static int my_checker(const struct lxc_arguments* args)
 {
-	if (!args->statefile) {
+	if ((!args->statefile) && (args->statefd == -1)) {
 		lxc_error(args, "no statefile specified");
 		return -1;
 	}
 
+	if ((args->statefile) && (args->statefd != -1)) {
+		lxc_error(args, "--statefile AND --statefd abnormally set");
+		return -1;
+	}
+
 	return 0;
 }
 
 static int my_parser(struct lxc_arguments* args, int c, char* arg)
 {
 	switch (c) {
-	case 'd': args->statefile = arg; break;
+	case 'S': args->statefile = arg; break;
 	case 'f': args->rcfile = arg; break;
 	case 'p': args->flags = LXC_FLAG_PAUSE; break;
 	case 's': return lxc_config_define_add(&defines, arg);
+	case 'd': {
+			int fd;
+			fd = lxc_arguments_str_to_int(args, arg);
+			if (fd < 0)
+				return -1;
+
+			args->statefd = fd;
+			break;
+		}
 	}
 
 	return 0;
 }
 
 static const struct option my_longopts[] = {
-	{"directory", required_argument, 0, 'd'},
+	{"statefile", required_argument, 0, 'S'},
+	{"statefd", required_argument, 0, 'd'},
 	{"rcfile", required_argument, 0, 'f'},
 	{"pause", no_argument, 0, 'p'},
 	{"define", required_argument, 0, 's'},
@@ -72,23 +88,28 @@ static const struct option my_longopts[] = {
 static struct lxc_arguments my_args = {
 	.progname = "lxc-restart",
 	.help     = "\
---name=NAME --directory STATEFILE\n\
+--name=NAME --statefile FILE\n\
 \n\
-lxc-restart restarts from STATEFILE the NAME container\n\
+lxc-restart restarts from FILE the NAME container\n\
 \n\
 Options :\n\
   -n, --name=NAME      NAME for name of the container\n\
-  -p, --pause          do not release the container after the restart\n\
-  -d, --directory=STATEFILE for name of statefile\n\
+  -p, --pause          do not unfreeze the container after the restart\n\
+  -S, --statefile=FILE read the container state from this file, or\n\
+  -d, --statefd=FD read the container state from this file descriptor\n\
   -f, --rcfile=FILE Load configuration file FILE\n\
   -s, --define KEY=VAL Assign VAL to configuration variable KEY\n",
 	.options  = my_longopts,
 	.parser   = my_parser,
 	.checker  = my_checker,
+
+	.statefd  = -1,
 };
 
 int main(int argc, char *argv[])
 {
+	int sfd = -1;
+	int ret;
 	char *rcfile = NULL;
 	struct lxc_conf *conf;
 
@@ -105,7 +126,10 @@ int main(int argc, char *argv[])
 	if (my_args.rcfile)
 		rcfile = (char *)my_args.rcfile;
 	else {
-		if (!asprintf(&rcfile, LXCPATH "/%s/config", my_args.name)) {
+		int rc;
+
+		rc = asprintf(&rcfile, LXCPATH "/%s/config", my_args.name);
+		if (rc == -1) {
 			SYSERROR("failed to allocate memory");
 			return -1;
 		}
@@ -131,6 +155,21 @@ int main(int argc, char *argv[])
 	if (lxc_config_define_load(&defines, conf))
 		return -1;
 
-	return lxc_restart(my_args.name, my_args.statefile, conf,
-			   my_args.flags);
+	if (my_args.statefd != -1)
+		sfd = my_args.statefd;
+
+#define OPEN_READ_MODE O_RDONLY | O_CLOEXEC | O_LARGEFILE
+	if (my_args.statefile) {
+		sfd = open(my_args.statefile, OPEN_READ_MODE, 0);
+		if (sfd < 0) {
+			ERROR("'%s' open failure : %m", my_args.statefile);
+			return sfd;
+		}
+	}
+
+	ret = lxc_restart(my_args.name, sfd, conf, my_args.flags);
+
+	if (my_args.statefile)
+		close(sfd);
+	return ret;
 }
