@@ -41,8 +41,10 @@
 #include <net/if.h>
 
 #include "log.h"
+#include "caps.h"
 #include "lxc.h"
 #include "conf.h"
+#include "cgroup.h"
 #include "utils.h"
 #include "config.h"
 #include "confile.h"
@@ -87,61 +89,22 @@ Options :\n\
 	.daemonize = 0,
 };
 
-static int save_tty(struct termios *tios)
-{
-	if (!isatty(0))
-		return 0;
-
-	if (tcgetattr(0, tios))
-		WARN("failed to get current terminal settings : %s",
-		     strerror(errno));
-
-	return 0;
-}
-
-static int restore_tty(struct termios *tios)
-{
-	struct termios current_tios;
-	void (*oldhandler)(int);
-	int ret;
-
-	if (!isatty(0))
-		return 0;
-
-	if (tcgetattr(0, &current_tios)) {
-		ERROR("failed to get current terminal settings : %s",
-		      strerror(errno));
-		return -1;
-	}
-
-	if (!memcmp(tios, &current_tios, sizeof(*tios)))
-		return 0;
-
-	oldhandler = signal(SIGTTOU, SIG_IGN);
-	ret = tcsetattr(0, TCSADRAIN, tios);
-	if (ret)
-		ERROR("failed to restore terminal attributes");
-	signal(SIGTTOU, oldhandler);
-
-	return ret;
-}
-
 int main(int argc, char *argv[])
 {
-	char *const *args;
 	int err = -1;
-	struct termios tios;
-
+	struct lxc_conf *conf;
+	char *const *args;
+	char *rcfile = NULL;
 	char *const default_args[] = {
 		"/sbin/init",
 		'\0',
 	};
 
-	char *rcfile = NULL;
-	struct lxc_conf *conf;
-
 	lxc_list_init(&defines);
 
+	if (lxc_caps_init())
+		return err;
+
 	if (lxc_arguments_parse(&my_args, argc, argv))
 		return err;
 
@@ -158,7 +121,10 @@ int main(int argc, char *argv[])
 	if (my_args.rcfile)
 		rcfile = (char *)my_args.rcfile;
 	else {
-		if (!asprintf(&rcfile, LXCPATH "/%s/config", my_args.name)) {
+		int rc;
+
+		rc = asprintf(&rcfile, LXCPATH "/%s/config", my_args.name);
+		if (rc == -1) {
 			SYSERROR("failed to allocate memory");
 			return err;
 		}
@@ -202,9 +168,9 @@ int main(int argc, char *argv[])
 			return err;
 		}
 
-		lxc_close_inherited_fd(0);
-		lxc_close_inherited_fd(1);
-		lxc_close_inherited_fd(2);
+		close(0);
+		close(1);
+		close(2);
 
 		if (my_args.log_file) {
 			open(my_args.log_file, O_WRONLY | O_CLOEXEC);
@@ -213,11 +179,18 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	save_tty(&tios);
-
 	err = lxc_start(my_args.name, args, conf);
 
-	restore_tty(&tios);
+	/*
+	 * exec ourself, that requires to have all opened fd
+	 * with the close-on-exec flag set
+	 */
+	if (conf->reboot) {
+		INFO("rebooting container");
+		execvp(argv[0], argv);
+		SYSERROR("failed to exec");
+		err = -1;
+	}
 
 	return err;
 }
